import {
  isBoolean,
  isFunction,
  isInteger,
  isNumber,
  isObject,
  isString
} from "./chunk-MSASB4XB.js";
import {
  first
} from "./chunk-UOAPXGA4.js";

// src/common/schema/schema.ts
var TypeClass = class {
  constructor(name, check) {
    this.type = name;
    this._check = check ?? (() => true);
  }
  optional() {
    this._optional = true;
    return this;
  }
  default(value) {
    this._default = value;
    return this;
  }
  parse(obj) {
    if (obj == null) {
      if (this._default != null) {
        if (isFunction(this._default))
          obj = this._default();
        else
          obj = this._default;
      }
    }
    if (obj == null && this._optional === true)
      return void 0;
    if (obj == null)
      throw new Error("cannot be undefined");
    if (!this._check || this._check(obj))
      return obj;
    throw new Error("wrong value");
  }
  map(obj, fn) {
    return fn.call(this, obj, this) ?? obj;
  }
  props(props) {
    this._props = props;
    return this;
  }
};
var TypeGeneric = class extends TypeClass {
};
function generic(type, opt) {
  return new TypeGeneric(type, opt?._check);
}
var TypeStringClass = class extends TypeClass {
  pattern(rx) {
    return this;
  }
};
function string() {
  return new TypeStringClass("string", isString);
}
function number() {
  return generic("number", {
    _check: isNumber
  });
}
var float = number;
function int() {
  return generic("int", {
    _check: isInteger
  });
}
function boolean() {
  return generic("boolean", {
    _check: isBoolean
  });
}
var TypeObjectClass = class extends TypeClass {
  constructor(obj) {
    super("object", () => true);
    this._object = obj;
  }
  parse(obj) {
    if (obj == null && this._optional === true)
      return void 0;
    const newObj = {};
    if (!isObject(obj))
      return new Error("expected object input");
    if (!isObject(this._object))
      return new Error("expected object definition");
    for (const [key, info] of Object.entries(this._object)) {
      const value = info.parse(obj[key]);
      if (value !== void 0)
        newObj[key] = value;
    }
    return newObj;
  }
  map(obj, fn) {
    const result = fn.call(this, obj, this);
    if (result !== void 0)
      return result;
    const newObj = {};
    if (obj) {
      for (const [key, info] of Object.entries(this._object ?? {})) {
        const value = info.map(obj[key], fn);
        if (value !== void 0)
          newObj[key] = value;
      }
    }
    return newObj;
  }
};
function object(tobj) {
  return new TypeObjectClass(tobj);
}
function union(options) {
  return generic(first(options)?.type ?? "any", {
    // _union: options,
    // _check(obj) {
    //   return this._union?.some(t => t._check(obj)) ?? true
    // },
  });
}
function literal(value) {
  return generic("string", {
    _check: (v) => v === value
  });
}
function stringLiterals(value) {
  return generic("string", {
    _check: (v) => value.includes(v)
  });
}
function tuple(items) {
  return generic("tuple", {
    _check: (v) => items.every((item, i) => item._check(v[i]))
  });
}
function func(args, ret) {
  return generic("function", {
    _check: (v) => isFunction(v)
  });
}

export {
  TypeClass,
  string,
  number,
  float,
  int,
  boolean,
  TypeObjectClass,
  object,
  union,
  literal,
  stringLiterals,
  tuple,
  func
};
//# sourceMappingURL=chunk-QY5DHPIN.js.map